searchNodes=[{"doc":"Contains the core functionality to work with Vaxin.Vaxin at its core is a data validator combinator library. It tries to solve the problem of validating the shape and content of some data (most useful when such data come from an external source) and of conforming those data to arbitrary formats.Vaxin is based on the concept of validators: a validator is something that knows how to validate a term and transform it to something else if necessary. A good example of a validator could be something that validates that a term is a string representation of an integer and that converts such string to the represented integer.ValidatorsA validator is a function that takes one argument and returns either:{:ok, transformed} - indicating the validation has succeeded (the input term is considered valid) and transformed is the conformed value for the input term.{:error, reason} - indicating means the validation has failed (the input term is invalid). reason can be a string representing the error message or a Vaxin.Error. Note that validate/2 will eventually wrap the error message into a Vaxin.Error.true - indicating the validation has succeeded. It has the same effect as {:ok, transformed}, but it can be used when the transformed value is the same as the input value. This is useful for &quot;predicate&quot; validators (functions that take one argument and return a boolean).false - it means validation failed. It is the same as {:error, reason}, except the reason only mentions that a &quot;predicate failed&quot;.Returning a boolean value is supported so that existing predicate functions can be used as validators without modification. Examples of such functions are type guards (is_binary/1 or is_list/1), functions like String.valid?/1, and many others.The concept of validators is very powerful as they can be easily combined: for example, the Vaxin.all_of/1 function takes a list of validators and returns a validator that passes if all of the given validators pass. Vaxin provides both &quot;basic&quot; validators as well as validator combinators.Built-in validators On top of powerful built-in Elixir predicate functions, Vaxin also provides a few built-in validators. You might notice that they are very similar to the Ecto.Changeset API. The intention is to enable developers who are familiar with Ecto to be immediately productive with Vaxin. However, there is a few fundamental difference between two libraries:Vaxin built-in validators take in options and return a validator which can be used with Vaxin.validate/2 later.Vaxin does not have the concept of &quot;empty&quot; values. nil or empty strings are treated the same way as other Elixir data. Consider the following example: nil will be validated with Vaxin while Ecto would skip it.iex&gt; import Vaxin iex&gt; validator = validate_number(greater_than: 0) iex&gt; {:error, error} = validate(validator, nil) iex&gt; Exception.message(error) &quot;must be a number&quot;ExamplesLet's say S.H.I.E.L.D are looking for a replacement for Captain America and receive thousands of applications, they could use Vaxin to build a profile validator.iex&gt; import Vaxin iex&gt; iex&gt; age_validator = ...&gt; validate_number( ...&gt; &amp;is_integer/1, ...&gt; greater_than: 18, ...&gt; message: &quot;is too young to be a superhero&quot; ...&gt; ) iex&gt; iex&gt; superpower_validator = ...&gt; validate_inclusion( ...&gt; &amp;is_binary/1, ...&gt; [&quot;fly&quot;, &quot;strength&quot;, &quot;i-can-do-this-all-day&quot;], ...&gt; message: &quot;is unfortunately not the super-power we are looking for&quot; ...&gt; ) iex&gt; superhero_validator = ...&gt; (&amp;is_map/1) ...&gt; |&gt; validate_key(&quot;age&quot;, :required, age_validator) ...&gt; |&gt; validate_key(&quot;superpower&quot;, :required, superpower_validator) iex&gt; iex&gt; peter_parker = %{&quot;age&quot; =&gt; 16, &quot;superpower&quot; =&gt; &quot;speed&quot;} iex&gt; {:error, error} = Vaxin.validate(superhero_validator, peter_parker) iex&gt; Exception.message(error) ~s(&quot;age&quot; is too young to be a superhero) iex&gt; iex&gt; falcon = %{&quot;age&quot; =&gt; 40, &quot;superpower&quot; =&gt; &quot;fly&quot;} iex&gt; Vaxin.validate(superhero_validator, falcon) {:ok, %{&quot;age&quot; =&gt; 40, &quot;superpower&quot; =&gt; &quot;fly&quot;}}","ref":"Vaxin.html","title":"Vaxin","type":"module"},{"doc":"Returns a validator that passes when all the given validators pass.Examplesiex&gt; validator = Vaxin.all_of([&amp;is_integer/1, &amp;(&amp;1 &gt;= 1)]) iex&gt; Vaxin.validate(validator, 1) {:ok, 1} iex&gt; {:error, %Vaxin.Error{message: &quot;is invalid&quot;}} = Vaxin.validate(validator, 0)","ref":"Vaxin.html#all_of/1","title":"Vaxin.all_of/1","type":"function"},{"doc":"Combines validator1 with validator2. Note that validator2 will only be executed if validator1 succeeds.Examplesiex&gt; validator = Vaxin.combine(&amp;is_integer/1, &amp;(&amp;1 &gt;= 1)) iex&gt; Vaxin.validate(validator, 1) {:ok, 1} iex&gt; {:error, %Vaxin.Error{message: &quot;is invalid&quot;}} = Vaxin.validate(validator, 0)","ref":"Vaxin.html#combine/2","title":"Vaxin.combine/2","type":"function"},{"doc":"Returns a validator that always passes. It is useful placing in the beginning of the validator chain.Examplesiex&gt; validator = Vaxin.noop() |&gt; Vaxin.validate_inclusion([:foo, &quot;foo&quot;]) iex&gt; Vaxin.validate(validator, :foo) {:ok, :foo} iex&gt; Vaxin.validate(validator, &quot;foo&quot;) {:ok, &quot;foo&quot;}","ref":"Vaxin.html#noop/0","title":"Vaxin.noop/0","type":"function"},{"doc":"Puts the given value under key unless the entry key already exists in mapExamplesiex&gt; import Vaxin iex&gt; validator = validate_key(&amp;is_map/1, &quot;foo&quot;, :optional, &amp;is_binary/1) iex&gt; validator = put_new(validator, &quot;foo&quot;, &quot;bar&quot;) iex&gt; validate(validator, %{}) {:ok, %{&quot;foo&quot; =&gt; &quot;bar&quot;}} iex&gt; validator = validate_key(&amp;is_map/1, &quot;foo&quot;, :optional, &amp;is_binary/1) iex&gt; validator = put_new(validator, &quot;foo&quot;, &quot;foo&quot;) iex&gt; validate(validator, %{&quot;foo&quot; =&gt; &quot;bar&quot;}) {:ok, %{&quot;foo&quot; =&gt; &quot;bar&quot;}}","ref":"Vaxin.html#put_new/3","title":"Vaxin.put_new/3","type":"function"},{"doc":"Returns a validator that always passes and applies the given transformer.Examplesiex&gt; import Vaxin iex&gt; validator = transform(noop(), &amp;String.to_integer/1) iex&gt; validate(validator, &quot;1&quot;) {:ok, 1}","ref":"Vaxin.html#transform/2","title":"Vaxin.transform/2","type":"function"},{"doc":"Validates value against validator.Examplesiex&gt; Vaxin.validate(&amp;is_atom/1, :foo) {:ok, :foo} iex&gt; Vaxin.validate(&amp;is_atom/1, &quot;foo&quot;) {:error, %Vaxin.Error{validator: &amp;is_atom/1, message: &quot;must be an atom&quot;, metadata: [kind: :is_atom]}}","ref":"Vaxin.html#validate/2","title":"Vaxin.validate/2","type":"function"},{"doc":"Combine combinator with a validator that validates every item in an enum against each_validator.Optionsskip_invalid? - (boolean) if true, skips all invalid items. Defaults to false.into - the collectable where the transformed values should end up in. Defaults to [].Examplesiex&gt; import Vaxin iex&gt; validator = validate_enum(&amp;is_list/1, &amp;is_integer/1) iex&gt; Vaxin.validate(validator, [1, 2]) {:ok, [1, 2]} iex&gt; {:error, error} = Vaxin.validate(validator, [1, &quot;2&quot;]) iex&gt; Exception.message(error) &quot;[1] must be an integer&quot; iex&gt; validator = validate_enum(&amp;is_list/1, &amp;is_integer/1, skip_invalid?: true) iex&gt; Vaxin.validate(validator, [1, &quot;2&quot;]) {:ok, [1]}","ref":"Vaxin.html#validate_enum/3","title":"Vaxin.validate_enum/3","type":"function"},{"doc":"Combines combinator with a validator that validates the term is excluded in permitted.Optionsmessage - the error message on failure. Defaults to &quot;is reserved&quot;.Examplesiex&gt; import Vaxin iex&gt; validator = validate_exclusion([&quot;foo&quot;, &quot;bar&quot;]) iex&gt; {:error, error} = validate(validator, &quot;foo&quot;) iex&gt; Exception.message(error) &quot;is reserved&quot;","ref":"Vaxin.html#validate_exclusion/3","title":"Vaxin.validate_exclusion/3","type":"function"},{"doc":"Combines combinator with a validator that validates the term matches the given regular expression.Optionsmessage - the error message when the format validator fails. Defaults to &quot;has invalid format&quot;.Examplesiex&gt; import Vaxin iex&gt; validator = validate_format(&amp;String.valid?/1, ~r/@/) iex&gt; validate(validator, &quot;foo@bar.com&quot;) {:ok, &quot;foo@bar.com&quot;}","ref":"Vaxin.html#validate_format/3","title":"Vaxin.validate_format/3","type":"function"},{"doc":"Combines combinator with a validator that validates the term is included in permitted.Optionsmessage - the error message on failure. Defaults to &quot;is invalid&quot;.Examplesiex&gt; import Vaxin iex&gt; validator = validate_inclusion([&quot;foo&quot;, &quot;bar&quot;]) iex&gt; validate(validator, &quot;foo&quot;) {:ok, &quot;foo&quot;}","ref":"Vaxin.html#validate_inclusion/3","title":"Vaxin.validate_inclusion/3","type":"function"},{"doc":"Combines combinator with a validator that checks the value of key in a map.Optionsmessage - the message on failure. Defaults to &quot;is required&quot; or the error returned by value_validator.Examplesiex&gt; tinyint_validator = validate_number(greater_than_or_equal_to: -128, less_than: 128, message: &quot;must be a tinyint&quot;) iex&gt; validator = Vaxin.validate_key(:id, :required, tinyint_validator) iex&gt; Vaxin.validate(validator, %{id: 1}) {:ok, %{id: 1}} iex&gt; {:error, error} = Vaxin.validate(validator, %{id: 129}) iex&gt; Exception.message(error) &quot;id must be a tinyint&quot;","ref":"Vaxin.html#validate_key/5","title":"Vaxin.validate_key/5","type":"function"},{"doc":"Combines combinator with a validator that validates the term as a number.Optionsless_than - the number must be less than this value.greater_than - the number must be greater than this value.less_than_or_equal_to - the number must be less than or equal to this value.greater_than_or_equal_to - the number must be greater than or equal to this value.equal_to - the number must be equal to this value.not_equal_to - the number must be not equal to this value.message - the error message when the number validator fails. Defaults to either:must be less than %{number}must be greater than %{number}must be less than or equal to %{number}must be greater than or equal to %{number}must be equal to %{number}must be not equal to %{number}Examplesiex&gt; validator = Vaxin.validate_number(greater_than: 1, less_than: 20) iex&gt; Vaxin.validate(validator, 10) {:ok, 10} iex&gt; {:error, error} = Vaxin.validate(validator, 20) iex&gt; Exception.message(error) &quot;must be less than 20&quot;","ref":"Vaxin.html#validate_number/2","title":"Vaxin.validate_number/2","type":"function"},{"doc":"Combines combinator with a validator that validates string length.Optionsexact - the length must be exact this value.min - the length must be greater than or equal to this value.max - the length must be less than or equal to this value.message - the message on failure. Defaults to either:must be %{length} byte(s)must be at least %{length} byte(s)must be at most %{length} byte(s)Examplesiex&gt; validator = Vaxin.validate_string_length(min: 1, max: 20) iex&gt; Vaxin.validate(validator, &quot;Hello World!&quot;) {:ok, &quot;Hello World!&quot;} iex&gt; {:error, error} = Vaxin.validate(validator, &quot;&quot;) iex&gt; Exception.message(error) &quot;must be at least 1 byte(s)&quot;","ref":"Vaxin.html#validate_string_length/2","title":"Vaxin.validate_string_length/2","type":"function"},{"doc":"","ref":"Vaxin.html#t:validator/0","title":"Vaxin.validator/0","type":"type"},{"doc":"","ref":"Vaxin.Error.html","title":"Vaxin.Error","type":"exception"},{"doc":"","ref":"Vaxin.Error.html#interpolate/2","title":"Vaxin.Error.interpolate/2","type":"function"},{"doc":"","ref":"Vaxin.Error.html#t:t/0","title":"Vaxin.Error.t/0","type":"type"},{"doc":"VaxinA validator combinator library for Elixir","ref":"readme.html","title":"Vaxin","type":"extras"},{"doc":"def deps() do [ {:vaxin, &quot;~&gt; 0.1.0&quot;} ] endFull documentation can be found on HexDocs","ref":"readme.html#installation","title":"Vaxin - Installation","type":"extras"},{"doc":"Vaxin at its core is a data validator combinator library. It tries to solve the problem of validating the shape and content of some data (most useful when such data come from an external source) and of conforming those data to arbitrary formats.Vaxin is based on the concept of validators: a validator is something that knows how to validate a term and transform it to something else if necessary. A good example of a validator could be something that validates that a term is a string representation of an integer and that converts such string to the represented integer.ValidatorsA validator is a function that takes one argument and returns either:{:ok, transformed} - indicating the validation has succeeded (the input term is considered valid) and transformed is the conformed value for the input term.{:error, reason} - indicating means the validation has failed (the input term is invalid). reason can be a string representing the error message or a Vaxin.Error. Note that validate/2 will eventually wrap the error message into a Vaxin.Error.true - indicating the validation has succeeded. It has the same effect as {:ok, transformed}, but it can be used when the transformed value is the same as the input value. This is useful for &quot;predicate&quot; validators (functions that take one argument and return a boolean).false - it means validation failed. It is the same as {:error, reason}, except the reason only mentions that a &quot;predicate failed&quot;.Returning a boolean value is supported so that existing predicate functions can be used as validators without modification. Examples of such functions are type guards (is_binary/1 or is_list/1), functions like String.valid?/1, and many others.The concept of validators is very powerful as they can be easily combined: for example, the Vaxin.all_of/1 function takes a list of validators and returns a validator that passes if all of the given validators pass. Vaxin provides both &quot;basic&quot; validators as well as validator combinators.Built-in validatorsOn top of powerful built-in Elixir predicate functions, Vaxin also provides a few built-in validators. You might notice that they are very similar to the Ecto.Changeset API. The intention is to enable developers who are familiar with Ecto to be immediately productive with Vaxin. However, there is a few fundamental difference between two libraries:Vaxin built-in validators take in options and return a validator which can be used with Vaxin.validate/2 later.Vaxin does not have the concept of &quot;empty&quot; values. nil or empty strings are treated the same way as other Elixir data.Consider the following example: nil will be validated with Vaxin while Ecto would skip it.iex&gt; import Vaxin iex&gt; validator = validate_number(greater_than: 0) iex&gt; {:error, error} = validate(validator, nil) iex&gt; Exception.message(error) &quot;must be a number&quot;","ref":"readme.html#usage","title":"Vaxin - Usage","type":"extras"},{"doc":"Let's say S.H.I.E.L.D are looking for a replacement for Captain America and receive thousands of applications, they could use Vaxin to build a profile validator.iex&gt; import Vaxin iex&gt; iex&gt; age_validator = ...&gt; validate_number( ...&gt; &amp;is_integer/1, ...&gt; greater_than: 18, ...&gt; message: &quot;is too young to be a superhero&quot; ...&gt; ) iex&gt; iex&gt; superpower_validator = ...&gt; validate_inclusion( ...&gt; &amp;is_binary/1, ...&gt; [&quot;fly&quot;, &quot;strength&quot;, &quot;i-can-do-this-all-day&quot;], ...&gt; message: &quot;is unfortunately not the super-power we are looking for&quot; ...&gt; ) iex&gt; superhero_validator = ...&gt; (&amp;is_map/1) ...&gt; |&gt; validate_key(&quot;age&quot;, :required, age_validator) ...&gt; |&gt; validate_key(&quot;superpower&quot;, :required, superpower_validator) iex&gt; iex&gt; peter_parker = %{&quot;age&quot; =&gt; 16, &quot;superpower&quot; =&gt; &quot;speed&quot;} iex&gt; {:error, error} = Vaxin.validate(superhero_validator, peter_parker) iex&gt; Exception.message(error) ~s(&quot;age&quot; is too young to be a superhero) iex&gt; iex&gt; falcon = %{&quot;age&quot; =&gt; 40, &quot;superpower&quot; =&gt; &quot;fly&quot;} iex&gt; Vaxin.validate(superhero_validator, falcon) {:ok, %{&quot;age&quot; =&gt; 40, &quot;superpower&quot; =&gt; &quot;fly&quot;}}","ref":"readme.html#examples","title":"Vaxin - Examples","type":"extras"},{"doc":"Vaxin was heavily inspired by Saul and Ecto.Changeset.","ref":"readme.html#credits","title":"Vaxin - Credits","type":"extras"},{"doc":"ISC","ref":"readme.html#license","title":"Vaxin - License","type":"extras"},{"doc":"CHANGELOG","ref":"changelog.html","title":"CHANGELOG","type":"extras"},{"doc":"Introduce transform/2 and put_new/3.","ref":"changelog.html#v0-2-0","title":"CHANGELOG - v0.2.0","type":"extras"},{"doc":"Initial release","ref":"changelog.html#v0-1-0","title":"CHANGELOG - v0.1.0","type":"extras"}]